@@ -15,6 +15,7 @@
 import fcntl
 import logging
 import os
+import re
 import select
 import shutil
 import socket
@@ -30,7 +31,10 @@ from nodepool import config as nodepool_config
 from nodepool import exceptions
 from nodepool import provider_manager
 from nodepool import stats
-from nodepool import zk
+from nodepool.zk import zookeeper as zk
+from nodepool.zk import ZooKeeperClient
+from nodepool.zk.components import BuilderComponent
+from nodepool.version import get_version_string
 
 
 MINS = 60
@@ -223,7 +227,7 @@ class CleanupWorker(BaseWorker):
         return False
 
     @staticmethod
-    def deleteLocalBuild(images_dir, image, build, log):
+    def deleteLocalBuild(images_dir, image_name, build_id, log):
         '''
         Remove expired image build from local disk.
 
@@ -244,12 +248,12 @@ class CleanupWorker(BaseWorker):
                 if e.errno != 2:    # No such file or directory
                     raise e
 
-        base = "-".join([image, build.id])
+        base = "-".join([image_name, build_id])
         files = DibImageFile.from_image_id(images_dir, base)
         if not files:
             return
 
-        log.info("Doing cleanup for %s:%s" % (image, build.id))
+        log.info("Doing cleanup for %s:%s" % (image_name, build_id))
 
         manifest_dir = None
 
@@ -268,9 +272,9 @@ class CleanupWorker(BaseWorker):
             if e.errno != 2:    # No such file or directory
                 raise e
 
-    def _deleteLocalBuild(self, image, build):
+    def _deleteLocalBuild(self, image_name, build_id):
         CleanupWorker.deleteLocalBuild(
-            self._config.images_dir, image, build, self.log)
+            self._config.images_dir, image_name, build_id, self.log)
 
     def _cleanupProvider(self, provider, image, build_id):
         all_uploads = self._zk.getUploads(image, build_id, provider.name)
@@ -374,14 +378,48 @@ class CleanupWorker(BaseWorker):
             except Exception:
                 self.log.exception("Exception cleaning up image %s:", image)
 
-    def _filterLocalBuilds(self, image, builds):
-        '''Return the subset of builds that are local'''
-        ret = []
-        for build in builds:
-            is_local = build.builder_id == self._builder_id
-            if is_local:
-                ret.append(build)
-        return ret
+        for local_build in self._getLocalBuilds():
+            try:
+                image_name, build_id = local_build
+                build = self._zk.getBuild(image_name, build_id)
+                if not build or build.state == zk.DELETING:
+                    self.log.info("Deleting on-disk build: "
+                                  "%s-%s", image_name, build_id)
+                    self._deleteLocalBuild(image_name, build_id)
+            except Exception:
+                self.log.exception("Exception cleaning up local build %s:",
+                                   local_build)
+
+    LOCAL_BUILDS_RE = re.compile(r'^(.*)-(\d+)\.(?:\w+)$')
+
+    def _getLocalBuilds(self):
+        seen = set()
+        for entry in os.scandir(self._config.images_dir):
+            m = self.LOCAL_BUILDS_RE.match(entry.name)
+            if m:
+                name_and_id = m.groups()
+                if name_and_id in seen:
+                    continue
+                seen.add(name_and_id)
+                yield name_and_id
+
+    def _emitBuildRequestStats(self):
+        '''Emit general build request stats
+
+        This runs in the cleanup worker because it's the least likely
+        thread to be stopped for hours on end.
+        '''
+        if not self._statsd:
+            return
+        count = 0
+        for image_name in self._zk.getImageNames():
+            request = self._zk.getBuildRequest(image_name)
+            if request and request.pending:
+                count += 1
+        pipeline = self._statsd.pipeline()
+        key = 'nodepool.image_build_requests'
+        pipeline.gauge(key, count)
+        pipeline.send()
 
     def _cleanupCurrentProviderUploads(self, provider, image, build_id):
         '''
@@ -432,21 +470,22 @@ class CleanupWorker(BaseWorker):
         # Get the list of all builds, then work from that so that we
         # have a consistent view of the data.
         all_builds = self._zk.getBuilds(image)
-        builds_to_keep = set([b for b in sorted(all_builds, reverse=True,
-                                                key=lambda y: y.state_time)
-                              if b.state == zk.READY][:2])
-        local_builds = set(self._filterLocalBuilds(image, all_builds))
         diskimage = self._config.diskimages.get(image)
-        if not diskimage and not local_builds:
-            # This builder is not configured to build this image and was not
-            # responsible for this image build so ignore it.
-            return
-        # Remove any local builds that are not in use.
-        if not diskimage or (diskimage and not diskimage.image_types):
-            builds_to_keep -= local_builds
+        if not diskimage:
+            # If the diskimage does not appear in the config at all,
+            # we interpret that as the user intending to remove it.
+            # If it appears in the config, even if it's not used by a
+            # provider, we will assume there is some other builder
+            # responsible for it, so we fall through to the else
+            # condition below and rely on data in ZK.
+            builds_to_keep = set()
             # TODO(jeblair): When all builds for an image which is not
             # in use are deleted, the image znode should be deleted as
             # well.
+        else:
+            builds_to_keep = set([b for b in sorted(all_builds, reverse=True,
+                                                    key=lambda y: y.state_time)
+                                  if b.state == zk.READY][:2])
 
         for build in all_builds:
             # Start by deleting any uploads that are no longer needed
@@ -497,13 +536,13 @@ class CleanupWorker(BaseWorker):
             for p in self._zk.getBuildProviders(image, build.id):
                 uploads += self._zk.getUploads(image, build.id, p)
 
-            # If we own this build, delete the local DIB files as soon as all
-            # provider uploads are in a deleting state. This prevents us from
-            # keeping local files around while we wait on clouds to remove
-            # the image on their side (which can be very slow).
+            # Mark the image as deleting (which will permit deleting
+            # local DIB files) as soon as all provider uploads are in
+            # a deleting state. This prevents us from keeping local
+            # files around while we wait on clouds to remove the image
+            # on their side (which can be very slow).
             all_deleting = all(map(lambda x: x.state == zk.DELETING, uploads))
-            is_local = build.builder_id == self._builder_id
-            if (not uploads or all_deleting) and is_local:
+            if (not uploads or all_deleting):
                 # If we got here, it's either an explicit delete (user
                 # activated), or we're deleting because we have newer
                 # images.  We're about to start deleting files, so
@@ -515,16 +554,10 @@ class CleanupWorker(BaseWorker):
                         build.state = zk.DELETING
                         self._zk.storeBuild(image, build, build.id)
 
-                self._deleteLocalBuild(image, build)
-
             if not uploads:
-                if is_local:
-                    # Only remove the db build record from the builder that
-                    # built this image. This prevents us from removing the db
-                    # record without first removing the local image files.
-                    if not self._zk.deleteBuild(image, build.id):
-                        self.log.error("Unable to delete build %s because"
-                                       " uploads still remain.", build)
+                if not self._zk.deleteBuild(image, build.id):
+                    self.log.error("Unable to delete build %s because"
+                                   " uploads still remain.", build)
 
     def run(self):
         '''
@@ -566,6 +599,7 @@ class CleanupWorker(BaseWorker):
         self._config = new_config
 
         self._cleanup()
+        self._emitBuildRequestStats()
 
 
 class BuildWorker(BaseWorker):
@@ -575,6 +609,11 @@ class BuildWorker(BaseWorker):
                                           interval, zk)
         self.log = logging.getLogger("nodepool.builder.BuildWorker.%s" % name)
         self.name = 'BuildWorker.%s' % name
+        self._lost_zk_connection = False
+        zk.client.on_connection_lost_listeners.append(self._onConnectionLost)
+
+    def _onConnectionLost(self):
+        self._lost_zk_connection = True
 
     def _getBuildLogRoot(self, name):
         log_dir = self._config.build_log_dir
@@ -675,6 +714,7 @@ class BuildWorker(BaseWorker):
 
         :returns: The updated ImageBuild data structure.
         '''
+        self._lost_zk_connection = False
         data = zk.ImageBuild()
         data.state = zk.BUILDING
         data.builder_id = self._builder_id
@@ -707,7 +747,7 @@ class BuildWorker(BaseWorker):
                 bnum, diskimage.name)
             data.id = bnum
             CleanupWorker.deleteLocalBuild(
-                self._config.images_dir, diskimage.name, data, self.log)
+                self._config.images_dir, diskimage.name, data.id, self.log)
             data.state = zk.FAILED
             return data
 
@@ -929,9 +969,8 @@ class BuildWorker(BaseWorker):
         if self._statsd:
             pipeline = self._statsd.pipeline()
 
-        if self._zk.didLoseConnection:
+        if self._lost_zk_connection:
             self.log.info("ZooKeeper lost while building %s" % diskimage.name)
-            self._zk.resetLostFlag()
             build_data.state = zk.FAILED
         elif p.returncode or did_timeout:
             self.log.info(
@@ -1083,7 +1122,8 @@ class UploadWorker(BaseWorker):
                 "Could not find matching provider image for %s" % image_name
             )
 
-        meta = provider_image.meta.copy()
+        meta = self._config.diskimages[image_name].meta.copy()
+        meta.update(provider_image.meta)
         meta['nodepool_build_id'] = build_id
         meta['nodepool_upload_id'] = upload_id
 
@@ -1362,7 +1402,7 @@ class NodePoolBuilder(object):
         config = nodepool_config.loadConfig(self._config_path)
         if self._secure_path:
             nodepool_config.loadSecureConfig(config, self._secure_path)
-        if not config.zookeeper_servers.values():
+        if not config.zookeeper_servers:
             raise RuntimeError('No ZooKeeper servers specified in config.')
         if not config.images_dir:
             raise RuntimeError('No images-dir specified in config.')
@@ -1392,14 +1432,21 @@ class NodePoolBuilder(object):
             builder_id = self._getBuilderID(builder_id_file)
 
             # All worker threads share a single ZooKeeper instance/connection.
-            self.zk = zk.ZooKeeper(enable_cache=False)
-            self.zk.connect(
-                list(self._config.zookeeper_servers.values()),
+            self.zk_client = ZooKeeperClient(
+                self._config.zookeeper_servers,
                 tls_cert=self._config.zookeeper_tls_cert,
                 tls_key=self._config.zookeeper_tls_key,
                 tls_ca=self._config.zookeeper_tls_ca,
                 timeout=self._config.zookeeper_timeout,
             )
+            self.zk_client.connect()
+            self.zk = zk.ZooKeeper(self.zk_client, enable_cache=False)
+
+            hostname = socket.gethostname()
+            self.component_info = BuilderComponent(
+                self.zk_client, hostname,
+                version=get_version_string())
+            self.component_info.register()
 
             self.log.debug('Starting listener for build jobs')
 
