@@ -24,7 +24,7 @@ import math
 import os
 import voluptuous as v
 
-from nodepool import zk
+from nodepool.zk import zookeeper as zk
 from nodepool import exceptions
 from nodepool.logconfig import get_annotated_logger
 
@@ -160,6 +160,12 @@ class Provider(ProviderNotifications):
     The class or instance attribute **name** must be provided as a string.
 
     """
+
+    # This is only intended for the static driver.  It indicates
+    # whether we should pause the provider when we are at quota.  This
+    # should almost always be true.
+    PAUSE_AT_QUOTA = True
+
     def __init__(self, *args, **kw):
         super().__init__(*args, **kw)
 
@@ -189,6 +195,17 @@ class Provider(ProviderNotifications):
         """
         pass
 
+    @abc.abstractmethod
+    def idle(self):
+        """Idle the provider
+
+        This is called before stop(). Providers should use this as a signal
+        to idle themselves and stop performing any actions that may interfere
+        with a new version of this provider starting up.
+
+        """
+        pass
+
     @abc.abstractmethod
     def join(self):
         """Wait for provider to finish
@@ -379,6 +396,10 @@ class NodeRequestHandler(NodeRequestHandlerNotifications,
         self.pool = self.pw.getPoolConfig()
         self.zk = self.pw.getZK()
         self.manager = self.pw.getProviderManager()
+        # We need the launcher_id attr
+        self.log = get_annotated_logger(logging.getLogger(
+            "nodepool.driver.NodeRequestHandler[%s]" % self.launcher_id),
+            event_id=self.request.event_id, node_request_id=self.request.id)
 
     @property
     def failed_nodes(self):
@@ -489,18 +510,23 @@ class NodeRequestHandler(NodeRequestHandlerNotifications,
                         self.log.debug(
                             "Declining node request because provider cannot"
                             " satisfy min-ready")
-                        self.decline_request()
-                        self._declinedHandlerCleanup()
+                        self.declineRequest()
                         return
 
                     self.log.info(
                         "Not enough quota remaining to satisfy request")
-                    if not self.paused:
-                        self.log.debug(
-                            "Pausing request handling to satisfy request")
-                    self.paused = True
                     self.zk.deleteOldestUnusedNode(self.provider.name,
                                                    self.pool.name)
+                    if self.manager.PAUSE_AT_QUOTA:
+                        if not self.paused:
+                            self.log.debug(
+                                "Pausing request handling to satisfy request")
+                        self.paused = True
+                    else:
+                        # Release the request so that we or another
+                        # provider can try again when the label quota
+                        # is available.
+                        self._declinedHandlerCleanup()
                     return
 
                 if self.paused:
@@ -539,22 +565,12 @@ class NodeRequestHandler(NodeRequestHandlerNotifications,
                 self._satisfied_types.add(ntype, node.id)
                 self.launch(node)
 
-    def _runHandler(self):
+    def getDeclinedReasons(self):
         '''
-        Main body for the node request handling.
+        Return a list of reasons to decline this request
         '''
-        self._setFromPoolWorker()
-
-        if self.provider is None or self.pool is None:
-            # If the config changed out from underneath us, we could now be
-            # an invalid provider and should stop handling this request.
-            raise Exception("Provider configuration missing")
-
-        # We have the launcher_id attr after _setFromPoolWorker() is called.
-        self.log = get_annotated_logger(logging.getLogger(
-            "nodepool.driver.NodeRequestHandler[%s]" % self.launcher_id),
-            event_id=self.request.event_id, node_request_id=self.request.id)
-
+        if not hasattr(self, 'provider'):
+            self._setFromPoolWorker()
         declined_reasons = []
         invalid_types = self._invalidNodeTypes()
 
@@ -567,12 +583,24 @@ class NodeRequestHandler(NodeRequestHandlerNotifications,
             declined_reasons.append('images are not available')
         elif not self.hasProviderQuota(self.request.node_types):
             declined_reasons.append('it would exceed quota')
+        return declined_reasons
 
+    def _runHandler(self):
+        '''
+        Main body for the node request handling.
+        '''
+        self._setFromPoolWorker()
+
+        if self.provider is None or self.pool is None:
+            # If the config changed out from underneath us, we could now be
+            # an invalid provider and should stop handling this request.
+            raise Exception("Provider configuration missing")
+
+        declined_reasons = self.getDeclinedReasons()
         if declined_reasons:
             self.log.info("Declining node request because %s",
                           ', '.join(declined_reasons))
-            self.decline_request()
-            self._declinedHandlerCleanup()
+            self.declineRequest()
             return
 
         if self.paused:
@@ -584,6 +612,10 @@ class NodeRequestHandler(NodeRequestHandlerNotifications,
 
         self._waitForNodeSet()
 
+    def declineRequest(self):
+        self._declineRequest()
+        self._declinedHandlerCleanup()
+
     def _declinedHandlerCleanup(self):
         """
         After declining a request, do necessary cleanup actions.
@@ -635,15 +667,15 @@ class NodeRequestHandler(NodeRequestHandlerNotifications,
 
         self.nodeset = []
 
-    def decline_request(self):
+    def _declineRequest(self):
         # Technically, this check to see if we've already declined it should
         # not be necessary. But if there is a bug (and there has been), we
         # want to make sure we don't continuously grow this array.
         if self.launcher_id not in self.request.declined_by:
             self.request.declined_by.append(self.launcher_id)
-        launchers = set([x.id for x in self.zk.getRegisteredLaunchers()])
-        if launchers.issubset(set(self.request.declined_by)):
-            # All launchers have declined it
+        launcher_pools = set([x.id for x in self.zk.getRegisteredPools()])
+        if launcher_pools.issubset(set(self.request.declined_by)):
+            # All launcher_pools have declined it
             self.log.debug("Failing declined node request")
             self.request.state = zk.FAILED
         else:
@@ -664,8 +696,7 @@ class NodeRequestHandler(NodeRequestHandlerNotifications,
             self.log.exception(
                 "Declining node request due to exception in "
                 "NodeRequestHandler:")
-            self.decline_request()
-            self._declinedHandlerCleanup()
+            self.declineRequest()
 
     def poll(self):
         if self.paused:
@@ -711,7 +742,8 @@ class NodeRequestHandler(NodeRequestHandlerNotifications,
 
         if self.failed_nodes:
             self.log.debug("Declining node request because nodes failed")
-            self.decline_request()
+            self._declineRequest()
+            # We perform our own cleanup
         elif aborted_nodes:
             # Because nodes are added to the satisfied types list before they
             # are ready we need to remove the aborted nodes again so they can
@@ -860,6 +892,8 @@ class ConfigPool(ConfigValue, metaclass=abc.ABCMeta):
         self.labels = {}
         self.max_servers = math.inf
         self.node_attributes = None
+        self.priority = None
+        self.ignore_provider_quota = False
 
     @classmethod
     def getCommonSchemaDict(self):
@@ -876,6 +910,7 @@ class ConfigPool(ConfigValue, metaclass=abc.ABCMeta):
         return {
             'max-servers': int,
             'node-attributes': dict,
+            'priority': int,
         }
 
     @abc.abstractmethod
@@ -894,6 +929,7 @@ class ConfigPool(ConfigValue, metaclass=abc.ABCMeta):
         '''
         self.max_servers = pool_config.get('max-servers', math.inf)
         self.node_attributes = pool_config.get('node-attributes')
+        self.priority = pool_config.get('priority', None)
 
 
 class DriverConfig(ConfigValue):
@@ -913,13 +949,15 @@ class ProviderConfig(ConfigValue, metaclass=abc.ABCMeta):
         self.driver = DriverConfig()
         self.driver.name = provider.get('driver', 'openstack')
         self.max_concurrency = provider.get('max-concurrency', -1)
+        self.priority = provider.get('priority', None)
 
     @classmethod
     def getCommonSchemaDict(self):
         return {
             v.Required('name'): str,
             'driver': str,
-            'max-concurrency': int
+            'max-concurrency': int,
+            'priority': int,
         }
 
     @property
