@@ -12,6 +12,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import functools
 import json
 import logging
 import math
@@ -52,7 +53,7 @@ def generate_password():
         if ((set(string.ascii_lowercase) & set(chars)) and
             (set(string.ascii_uppercase) & set(chars)) and
             (set(string.digits) & set(chars))):
-            return(''.join(chars))
+            return ''.join(chars)
 
 
 class AzureInstance(statemachine.Instance):
@@ -83,8 +84,12 @@ class AzureInstance(statemachine.Instance):
 
         self.interface_ip = (self.public_ipv4 or self.public_ipv6 or
                              self.private_ipv4 or self.private_ipv6)
+        self.cloud = 'Azure'
         self.region = vm['location']
-        self.az = ''
+        if len(vm.get('zones', [])) > 0:
+            self.az = vm['zones'][0]
+        else:
+            self.az = ''
 
     def getQuotaInformation(self):
         return quota_info_from_sku(self.sku)
@@ -169,8 +174,9 @@ class AzureCreateStateMachine(statemachine.StateMachine):
     COMPLETE = 'complete'
 
     def __init__(self, adapter, hostname, label, image_external_id,
-                 metadata, retries):
+                 metadata, retries, request, log):
         super().__init__()
+        self.log = log
         self.adapter = adapter
         self.retries = retries
         self.attempts = 0
@@ -178,6 +184,11 @@ class AzureCreateStateMachine(statemachine.StateMachine):
         self.image_reference = None
         self.metadata = metadata
         self.tags = label.tags.copy() or {}
+        for k, v in label.dynamic_tags.items():
+            try:
+                self.tags[k] = v.format(request=request.getSafeAttributes())
+            except Exception:
+                self.log.exception("Error formatting tag %s", k)
         self.tags.update(metadata)
         self.hostname = hostname
         self.label = label
@@ -304,6 +315,12 @@ class AzureAdapter(statemachine.Adapter):
     log = logging.getLogger("nodepool.driver.azure.AzureAdapter")
 
     def __init__(self, provider_config):
+        # Wrap these instance methods with a per-instance LRU cache so
+        # that we don't leak memory over time when the adapter is
+        # occasionally replaced.
+        self._getImage = functools.lru_cache(maxsize=None)(
+            self._getImage)
+
         self.provider = provider_config
         self.resource_group = self.provider.resource_group
         self.resource_group_location = self.provider.resource_group_location
@@ -329,9 +346,11 @@ class AzureAdapter(statemachine.Adapter):
         self._getSKUs()
 
     def getCreateStateMachine(self, hostname, label,
-                              image_external_id, metadata, retries, log):
+                              image_external_id, metadata, retries,
+                              request, log):
         return AzureCreateStateMachine(self, hostname, label,
-                                       image_external_id, metadata, retries)
+                                       image_external_id, metadata,
+                                       retries, request, log)
 
     def getDeleteStateMachine(self, external_id, log):
         return AzureDeleteStateMachine(self, external_id)
@@ -546,7 +565,7 @@ class AzureAdapter(statemachine.Adapter):
                     self.skus[key] = sku
         self.log.debug("Done querying compute SKUs")
 
-    @cachetools.func.ttl_cache(maxsize=0, ttl=(24 * 60 * 60))
+    # This method is wrapped with an LRU cache in the constructor.
     def _getImage(self, image_name):
         with self.rate_limiter:
             return self.azul.images.get(self.resource_group, image_name)
