@@ -132,6 +132,7 @@ class IBMVPCInstance(statemachine.Instance):
             self.public_ipv4 = fip['address']
 
         self.interface_ip = self.public_ipv4 or self.private_ipv4
+        self.cloud = 'IBM'
         self.region = provider.region
         self.az = vm['zone']['name']
 
@@ -379,9 +380,11 @@ class IBMVPCAdapter(statemachine.Adapter):
         return authenticator
 
     def getCreateStateMachine(self, hostname, label,
-                              image_external_id, metadata, retries, log):
+                              image_external_id, metadata, retries,
+                              request, log):
         return IBMVPCCreateStateMachine(self, hostname, label,
-                                        image_external_id, metadata, retries)
+                                        image_external_id, metadata,
+                                        retries)
 
     def getDeleteStateMachine(self, external_id, log):
         return IBMVPCDeleteStateMachine(self, external_id)
