@@ -16,10 +16,11 @@ import math
 import logging
 import json
 import time
+import threading
 
 from nodepool.driver.utils import QuotaInformation, RateLimiter
 from nodepool.driver import statemachine
-from nodepool import zk
+from nodepool.zk import zookeeper as zk
 
 
 """ This driver behaves like a static driver execpt that the backing
@@ -133,6 +134,7 @@ class MetastaticInstance(statemachine.Instance):
             'node': node.id,
         }
         self.external_id = node.id
+        self.slot = slot
 
     def getQuotaInformation(self):
         return QuotaInformation(instances=1)
@@ -245,6 +247,7 @@ class BackingNodeRecord:
         idx = self.allocated_nodes.index(node_id)
         self.allocated_nodes[idx] = None
         self.last_used = time.time()
+        return idx
 
     def backsNode(self, node_id):
         return node_id in self.allocated_nodes
@@ -262,15 +265,19 @@ class MetastaticAdapter(statemachine.Adapter):
         # The requestor id
         self.my_id = f'NodePool:metastatic:{self.provider.name}'
         # On startup we need to recover our state from the ZK db, this
-        # flag ensures we only do that once.
+        # flag ensures we only do that once.  We also can't do it here
+        # because we have to wait for ZK to be initialized which may
+        # be well after this point.
         self.performed_init = False
+        self.init_lock = threading.Lock()
 
     @property
     def zk(self):
         return self._provider._zk
 
     def getCreateStateMachine(self, hostname, label,
-                              image_external_id, metadata, retries, log):
+                              image_external_id, metadata, retries,
+                              request, log):
         return MetastaticCreateStateMachine(self, hostname, label,
                                             image_external_id, metadata,
                                             retries)
@@ -287,8 +294,14 @@ class MetastaticAdapter(statemachine.Adapter):
             self.backing_node_records.items():
             for bnr in backing_node_records[:]:
                 label_config = self.provider._getLabel(bnr.label_name)
+                if label_config:
+                    grace_time = label_config.grace_time
+                else:
+                    # The label doesn't exist in our config any more,
+                    # it must have been removed.
+                    grace_time = 0
                 if (bnr.isEmpty() and
-                    now - bnr.last_used > label_config.grace_time):
+                    now - bnr.last_used > grace_time):
                     self.log.info("Backing node %s has been idle for "
                                   "%s seconds, releasing",
                                   bnr.node_id, now - bnr.last_used)
@@ -318,8 +331,12 @@ class MetastaticAdapter(statemachine.Adapter):
     # Local implementation below
 
     def _init(self):
-        if self.performed_init:
-            return
+        with self.init_lock:
+            if self.performed_init:
+                return
+            self._init_inner()
+
+    def _init_inner(self):
         self.log.debug("Performing init")
         # Find backing nodes
         backing_node_map = {}
@@ -381,9 +398,11 @@ class MetastaticAdapter(statemachine.Adapter):
                 label.name, label.max_parallel_jobs)
             backing_node_record.request_id = req.id
             self._addBackingNode(label.name, backing_node_record)
+        backing_node_log = (backing_node_record.node_id or
+                            f'request {backing_node_record.request_id}')
         slot = backing_node_record.allocateSlot(node_id)
         self.log.info("Assigned node %s to backing node %s slot %s",
-                      node_id, backing_node_record.node_id, slot)
+                      node_id, backing_node_log, slot)
         return backing_node_record, slot
 
     def _addBackingNode(self, label_name, backing_node_record):
@@ -396,7 +415,10 @@ class MetastaticAdapter(statemachine.Adapter):
             self.backing_node_records.items():
             for bn in backing_node_records:
                 if bn.backsNode(node_id):
-                    bn.deallocateSlot(node_id)
+                    slot = bn.deallocateSlot(node_id)
+                    self.log.info(
+                        "Unassigned node %s from backing node %s slot %s",
+                        node_id, bn.node_id, slot)
                     return
 
     def _checkBackingNodeRequests(self):
