@@ -17,7 +17,7 @@ import logging
 from kazoo import exceptions as kze
 
 from nodepool import exceptions
-from nodepool import zk
+from nodepool.zk import zookeeper as zk
 from nodepool.driver.utils import NodeLauncher
 from nodepool.driver import NodeRequestHandler
 
@@ -50,6 +50,7 @@ class OpenshiftLauncher(NodeLauncher):
         self.node.shell_type = self.label.shell_type
         # NOTE: resource access token may be encrypted here
         self.node.connection_port = resource
+        self.node.cloud = self.provider_config.context
         self.zk.storeNode(self.node)
         self.log.info("Resource %s is ready", project)
 
