@@ -175,7 +175,9 @@ class ProviderPool(ConfigPool):
                                                  False))
             pl.volume_size = label.get('volume-size', 50)
             pl.instance_properties = label.get('instance-properties',
-                                               None)
+                                               {})
+            pl.dynamic_instance_properties = label.get(
+                'dynamic-instance-properties', {})
             pl.userdata = label.get('userdata', None)
             pl.networks = label.get('networks', self.networks)
             pl.host_key_checking = label.get(
@@ -265,16 +267,6 @@ class OpenStackProviderConfig(ProviderConfig):
             # This dict is expanded and used as custom properties when
             # the image is uploaded.
             i.meta = image.get('meta', {})
-            # 5 elements, and no key or value can be > 255 chars
-            # per Nova API rules
-            if i.meta:
-                if len(i.meta) > 5 or \
-                   any([len(k) > 255 or len(v) > 255
-                        for k, v in i.meta.items()]):
-                    # soft-fail
-                    # self.log.error("Invalid metadata for %s; ignored"
-                    #               % i.name)
-                    i.meta = {}
 
         for image in self.provider.get('cloud-images', []):
             i = ProviderCloudImage()
@@ -329,6 +321,7 @@ class OpenStackProviderConfig(ProviderConfig):
             'boot-from-volume': bool,
             'volume-size': int,
             'instance-properties': dict,
+            'dynamic-instance-properties': dict,
             'userdata': str,
             'networks': [str],
             'host-key-checking': bool,
