@@ -28,7 +28,7 @@ from nodepool.driver.utils import QuotaInformation, QuotaSupport
 from nodepool.driver.utils import NodeDeleter
 from nodepool import stats
 from nodepool import version
-from nodepool import zk
+from nodepool.zk import zookeeper as zk
 
 # Import entire module to avoid partial-loading, circular import
 from nodepool.driver.openstack import handler
@@ -69,6 +69,9 @@ class OpenStackProvider(Provider, QuotaSupport):
         self.running = False
         self._server_list_watcher_stop_event.set()
 
+    def idle(self):
+        pass
+
     def join(self):
         self._server_list_watcher.join()
 
