@@ -30,10 +30,12 @@ from nodepool import exceptions
 from nodepool import provider_manager
 from nodepool import stats
 from nodepool import config as nodepool_config
-from nodepool import zk
+from nodepool.zk import zookeeper as zk
+from nodepool.zk import ZooKeeperClient
+from nodepool.zk.components import LauncherComponent, PoolComponent
 from nodepool.driver.utils import QuotaInformation, QuotaSupport
 from nodepool.logconfig import get_annotated_logger
-from nodepool.version import version_info as npd_version_info
+from nodepool.version import get_version_string
 
 
 MINS = 60
@@ -79,6 +81,17 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
                                          uuid.uuid4().hex)
         stats.StatsReporter.__init__(self)
 
+    def getPriority(self):
+        pool = self.getPoolConfig()
+        provider = self.getProviderConfig()
+        if pool.priority is not None:
+            priority = pool.priority
+        elif provider.priority is not None:
+            priority = provider.priority
+        else:
+            priority = 100
+        return priority
+
     # ---------------------------------------------------------------
     # Private methods
     # ---------------------------------------------------------------
@@ -108,7 +121,7 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
         # become out of date as the loop progresses, but it should be
         # good enough to determine whether we should process requests
         # which express a preference for a specific provider.
-        launchers = self.zk.getRegisteredLaunchers()
+        launcher_pools = self.zk.getRegisteredPools()
 
         pm = self.getProviderManager()
         has_quota_support = isinstance(pm, QuotaSupport)
@@ -134,24 +147,6 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
             if not self.running:
                 return True
 
-            if self.paused_handler:
-                self.log.debug("Handler is now paused")
-                return True
-
-            # Get active threads for all pools for this provider
-            active_threads = sum([
-                w.activeThreads() for
-                w in self.nodepool.getPoolWorkers(self.provider_name)
-            ])
-
-            # Short-circuit for limited request handling
-            if (provider.max_concurrency > 0 and
-                    active_threads >= provider.max_concurrency):
-                self.log.debug("Request handling limited: %s active threads ",
-                               "with max concurrency of %s",
-                               active_threads, provider.max_concurrency)
-                return True
-
             req = self.zk.getNodeRequest(req.id)
             if not req:
                 continue
@@ -166,21 +161,37 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
 
             log = get_annotated_logger(self.log, event_id=req.event_id,
                                        node_request_id=req.id)
+            # Get the candidate launchers for these nodes
+            candidate_launcher_pools = set(
+                x for x in launcher_pools
+                if (set(x.supported_labels).issuperset(set(req.node_types)) and
+                    x.id not in req.declined_by)
+            )
             # Skip this request if it is requesting another provider
             # which is online
             if req.provider and req.provider != self.provider_name:
                 # The request is asking for a specific provider
-                candidate_launchers = set(
-                    x.id for x in launchers
+                launcher_pool_ids_for_provider = set(
+                    x.id for x in candidate_launcher_pools
                     if x.provider_name == req.provider
-                    and x.supported_labels.issuperset(req.node_types))
-                if candidate_launchers:
-                    # There is a launcher online which can satisfy the request
-                    if not candidate_launchers.issubset(set(req.declined_by)):
-                        # It has not yet declined the request, so yield to it.
-                        log.debug("Yielding request to provider %s %s",
-                                  req.provider, candidate_launchers)
-                        continue
+                )
+                if launcher_pool_ids_for_provider:
+                    # There is a launcher online which can satisfy the
+                    # request that has not yet declined the request,
+                    # so yield to it.
+                    log.debug("Yielding request to provider %s %s",
+                              req.provider, launcher_pool_ids_for_provider)
+                    continue
+
+            priority = self.getPriority()
+            launcher_pool_ids_with_higher_priority = set(
+                x.id for x in candidate_launcher_pools
+                if x.priority < priority and not x.paused
+            )
+            if launcher_pool_ids_with_higher_priority:
+                log.debug("Yielding request to higher priority providers %s",
+                          launcher_pool_ids_with_higher_priority)
+                continue
 
             if has_quota_support and not all(label_quota.get(l, math.inf) > 0
                                              for l in req.node_types):
@@ -198,10 +209,38 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
             if check_tenant_quota and not self._hasTenantQuota(req, pm):
                 # Defer request for it to be handled and fulfilled at a later
                 # run.
-                log.debug(
-                    "Deferring request because it would exceed tenant quota")
+                log.debug("Deferring request because it would "
+                          "exceed tenant quota")
                 continue
 
+            # Get a request handler to help decide whether we should
+            # accept the request, but we're still not sure yet.  We
+            # must lock the request before calling .run().
+            rh = pm.getRequestHandler(self, req)
+            reasons_to_decline = rh.getDeclinedReasons()
+
+            if self.paused_handler and not reasons_to_decline:
+                self.log.debug("Handler is paused, deferring request")
+                continue
+
+            # At this point, we are either unpaused, or we know we
+            # will decline the request.
+
+            if not reasons_to_decline:
+                # Get active threads for all pools for this provider
+                active_threads = sum([
+                    w.activeThreads() for
+                    w in self.nodepool.getPoolWorkers(self.provider_name)
+                ])
+                # Short-circuit for limited request handling
+                if (provider.max_concurrency > 0 and
+                        active_threads >= provider.max_concurrency):
+                    self.log.debug("Request handling limited: %s "
+                                   "active threads ",
+                                   "with max concurrency of %s",
+                                   active_threads, provider.max_concurrency)
+                    continue
+
             log.debug("Locking request")
             try:
                 self.zk.lockNodeRequest(req, blocking=False)
@@ -215,18 +254,26 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
                 log.debug("Request is in state %s", req.state)
                 continue
 
-            # Got a lock, so assign it
-            log.info("Assigning node request %s" % req)
+            if not reasons_to_decline:
+                # Got a lock, so assign it
+                log.info("Assigning node request %s" % req)
+                rh.run()
+            else:
+                log.info("Declining node request %s" % req)
+                rh.declineRequest()
+                continue
 
             if has_quota_support:
                 # Adjust the label quota so we don't accept more requests
                 # than we have labels available.
-                for label in req.node_types:
-                    with contextlib.suppress(KeyError):
-                        label_quota[label] -= 1
+                # Since nodes can have multiple other labels apart from the
+                # requested type, we need to adjust the quota for all labels
+                # of nodes that are allocated to the request.
+                for node in rh.nodeset:
+                    for node_type in node.type:
+                        with contextlib.suppress(KeyError):
+                            label_quota[node_type] -= 1
 
-            rh = pm.getRequestHandler(self, req)
-            rh.run()
             if rh.paused:
                 self.paused_handler = rh
             self.request_handlers.append(rh)
@@ -297,9 +344,10 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
             **self.nodepool.config.tenant_resource_limits[tenant_name])
 
         tenant_quota.subtract(used_quota)
-        log.debug("Current tenant quota: %s", tenant_quota)
+        log.debug("Current tenant quota for %s: %s", tenant_name, tenant_quota)
         tenant_quota.subtract(needed_quota)
-        log.debug("Predicted remaining tenant quota: %s", tenant_quota)
+        log.debug("Predicted remaining tenant quota for %s: %s",
+                  tenant_name, tenant_quota)
         return tenant_quota.non_negative()
 
     def _getUsedQuotaForTenant(self, tenant_name):
@@ -348,6 +396,23 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
     def run(self):
         self.running = True
 
+        # Make sure we're always registered with ZK
+        hostname = socket.gethostname()
+        self.component_info = PoolComponent(
+            self.zk.client, hostname,
+            version=get_version_string())
+        labels = set()
+        for prov_cfg in self.nodepool.config.providers.values():
+            labels.update(prov_cfg.getSupportedLabels())
+            self.component_info.content.update({
+                'id': self.launcher_id,
+                'provider_name': self.provider_name,
+                'supported_labels': list(labels),
+                'state': self.component_info.RUNNING,
+                'priority': self.getPriority(),
+            })
+        self.component_info.register()
+
         while self.running:
             try:
                 # Don't do work if we've lost communication with the ZK cluster
@@ -359,14 +424,11 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
                 if did_suspend:
                     self.log.info("ZooKeeper available. Resuming")
 
-                # Make sure we're always registered with ZK
-                launcher = zk.Launcher()
-                launcher.id = self.launcher_id
+                labels = set()
                 for prov_cfg in self.nodepool.config.providers.values():
-                    launcher.supported_labels.update(
-                        prov_cfg.getSupportedLabels())
-                launcher.provider_name = self.provider_name
-                self.zk.registerLauncher(launcher)
+                    labels.update(prov_cfg.getSupportedLabels())
+                self.component_info.supported_labels = list(labels)
+                self.component_info.priority = self.getPriority()
 
                 self.updateProviderLimits(
                     self.nodepool.config.providers.get(self.provider_name))
@@ -374,20 +436,27 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
                     self.nodepool.config.tenant_resource_limits)
 
                 if not self.paused_handler:
-                    while not self._assignHandlers():
-                        # _assignHandlers can take quite some time on a busy
-                        # system so sprinkle _removeCompletedHandlers in
-                        # between such that we have a chance to fulfill
-                        # requests that already have all nodes.
-                        self._removeCompletedHandlers()
+                    self.component_info.paused = False
                 else:
+                    self.component_info.paused = True
                     # If we are paused, one request handler could not
                     # satisfy its assigned request, so give it
                     # another shot. Unpause ourselves if it completed.
                     self.paused_handler.run()
                     if not self.paused_handler.paused:
                         self.paused_handler = None
+                        self.component_info.paused = False
 
+                # Regardless of whether we are paused, run
+                # assignHandlers.  It will only accept requests if we
+                # are unpaused, otherwise it will only touch requests
+                # we intend to decline.
+                while not self._assignHandlers():
+                    # _assignHandlers can take quite some time on a busy
+                    # system so sprinkle _removeCompletedHandlers in
+                    # between such that we have a chance to fulfill
+                    # requests that already have all nodes.
+                    self._removeCompletedHandlers()
                 self._removeCompletedHandlers()
             except Exception:
                 self.log.exception("Error in PoolWorker:")
@@ -407,7 +476,7 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
         '''
         self.log.info("%s received stop" % self.name)
         self.running = False
-        self.zk.deregisterLauncher(self.launcher_id)
+        self.component_info.unregister()
         self.stop_event.set()
 
 
@@ -434,7 +503,9 @@ class BaseCleanupWorker(threading.Thread):
             if did_suspend:
                 self.log.info("ZooKeeper available. Resuming")
 
+            self.log.debug('Starting cleanup')
             self._run()
+            self.log.debug('Finished cleanup')
             self._stop_event.wait(self._interval)
 
         self.log.info("Stopped")
@@ -962,26 +1033,34 @@ class NodePool(threading.Thread):
 
     def reconfigureZooKeeper(self, config):
         if self.config:
-            running = list(self.config.zookeeper_servers.values())
+            running = self.config.zookeeper_servers
         else:
             running = None
 
-        configured = list(config.zookeeper_servers.values())
+        configured = config.zookeeper_servers
         if running == configured:
             return
 
         if not self.zk and configured:
             self.log.debug("Connecting to ZooKeeper servers")
-            self.zk = zk.ZooKeeper()
-            self.zk.connect(configured,
-                            tls_cert=config.zookeeper_tls_cert,
-                            tls_key=config.zookeeper_tls_key,
-                            tls_ca=config.zookeeper_tls_ca,
-                            timeout=config.zookeeper_timeout,
-                            )
+            self.zk_client = ZooKeeperClient(
+                configured,
+                tls_cert=config.zookeeper_tls_cert,
+                tls_key=config.zookeeper_tls_key,
+                tls_ca=config.zookeeper_tls_ca,
+                timeout=config.zookeeper_timeout,
+            )
+            self.zk_client.connect()
+            self.zk = zk.ZooKeeper(self.zk_client)
+
+            hostname = socket.gethostname()
+            self.component_info = LauncherComponent(
+                self.zk_client, hostname,
+                version=get_version_string())
+            self.component_info.register()
         else:
             self.log.debug("Detected ZooKeeper server changes")
-            self.zk.resetHosts(configured)
+            self.zk_client.resetHosts(configured)
 
     def setConfig(self, config):
         self.config = config
@@ -1139,7 +1218,7 @@ class NodePool(threading.Thread):
         Start point for the NodePool thread.
         '''
         self.log.info("Nodepool launcher %s starting",
-                      npd_version_info.release_string())
+                      get_version_string())
         while not self._stopped:
             try:
                 self.updateConfig()
@@ -1153,6 +1232,8 @@ class NodePool(threading.Thread):
                 if did_suspend:
                     self.log.info("ZooKeeper available. Resuming")
 
+                if self.component_info.state != self.component_info.RUNNING:
+                    self.component_info.state = self.component_info.RUNNING
                 self.createMinReady()
 
                 if not self._cleanup_thread:
