@@ -20,7 +20,7 @@ from urllib import request
 from urllib.error import HTTPError
 
 from nodepool import tests
-from nodepool import zk
+from nodepool.zk import zookeeper as zk
 from nodepool.nodeutils import iterate_timeout
 
 
@@ -138,6 +138,60 @@ class TestWebApp(tests.DBTestCase):
                                        'formats': ['qcow2'],
                                        'state': 'ready'}, objs[0])
 
+    def test_image_status_json(self):
+        configfile = self.setup_config("node.yaml")
+        pool = self.useNodepool(configfile, watermark_sleep=1)
+        builder = self.useBuilder(configfile)
+        # Make sure we have enough time to test for the build request
+        # before it's processed by the build worker.
+        for worker in builder._build_workers:
+            worker._interval = 60
+
+        pool.start()
+        webapp = self.useWebApp(pool, port=0)
+        webapp.start()
+        port = webapp.server.socket.getsockname()[1]
+
+        self.waitForImage("fake-provider", "fake-image")
+        self.waitForNodes('fake-label')
+
+        req = request.Request(
+            "http://localhost:{}/image-status".format(port))
+        req.add_header("Accept", "application/json")
+
+        f = request.urlopen(req)
+        self.assertEqual(f.info().get("Content-Type"),
+                         "application/json")
+
+        data = f.read()
+        objs = json.loads(data.decode("utf8"))
+        self.assertDictContainsSubset({"image": "fake-image",
+                                       "paused": False,
+                                       "build_request": None}, objs[0])
+
+        self.zk.submitBuildRequest("fake-image")
+
+        webapp.cache.cache.clear()
+        f = request.urlopen(req)
+        data = f.read()
+        objs = json.loads(data.decode("utf8"))
+        self.assertDictContainsSubset({"image": "fake-image",
+                                       "paused": False,
+                                       "build_request": "pending"}, objs[0])
+
+        builder._janitor._emitBuildRequestStats()
+        self.assertReportedStat('nodepool.image_build_requests', '1', 'g')
+
+        webapp.cache.cache.clear()
+        with self.zk.imageBuildLock('fake-image', blocking=True, timeout=1):
+            f = request.urlopen(req)
+            data = f.read()
+
+        objs = json.loads(data.decode("utf8"))
+        self.assertDictContainsSubset({"image": "fake-image",
+                                       "paused": False,
+                                       "build_request": "building"}, objs[0])
+
     def test_node_list_json(self):
         configfile = self.setup_config('node.yaml')
         pool = self.useNodepool(configfile, watermark_sleep=1)
@@ -158,13 +212,15 @@ class TestWebApp(tests.DBTestCase):
                          'application/json')
         data = f.read()
         objs = json.loads(data.decode('utf8'))
+        # We don't check the value of 'locked' because we may get a
+        # cached value returned.
         self.assertDictContainsSubset({'id': '0000000000',
                                        'ipv6': '',
                                        'label': ['fake-label'],
-                                       'locked': 'unlocked',
                                        'provider': 'fake-provider',
                                        'public_ipv4': 'fake',
                                        'state': 'ready'}, objs[0])
+        self.assertTrue('locked' in objs[0])
         # specify valid node_id
         req = request.Request(
             "http://localhost:%s/node-list?node_id=%s" % (port,
@@ -175,13 +231,15 @@ class TestWebApp(tests.DBTestCase):
                          'application/json')
         data = f.read()
         objs = json.loads(data.decode('utf8'))
+        # We don't check the value of 'locked' because we may get a
+        # cached value returned.
         self.assertDictContainsSubset({'id': '0000000000',
                                        'ipv6': '',
                                        'label': ['fake-label'],
-                                       'locked': 'unlocked',
                                        'provider': 'fake-provider',
                                        'public_ipv4': 'fake',
                                        'state': 'ready'}, objs[0])
+        self.assertTrue('locked' in objs[0])
         # node_id not found
         req = request.Request(
             "http://localhost:%s/node-list?node_id=%s" % (port,
