@@ -14,11 +14,39 @@
 import testtools
 import time
 import uuid
+import socket
 
 from nodepool import exceptions as npe
 from nodepool import tests
-from nodepool import zk
+from nodepool.zk import zookeeper as zk
+from nodepool.zk.components import PoolComponent
+from nodepool.config import ZooKeeperConnectionConfig, buildZooKeeperHosts
 from nodepool.nodeutils import iterate_timeout
+from nodepool.version import get_version_string
+
+
+class TestComponentRegistry(tests.DBTestCase):
+
+    def test_pool_component(self):
+        hostname = socket.gethostname()
+        launcher = PoolComponent(
+            self.zk.client, hostname,
+            version=get_version_string())
+        launcher.content.update({
+            'id': "launcher-Poolworker.provider-main-" + uuid.uuid4().hex,
+            'provider_name': 'provider',
+            'supported_labels': [],
+            'state': launcher.RUNNING,
+        })
+        launcher.register()
+
+        launcher_pools = self.zk.getRegisteredPools()
+        self.assertEqual(1, len(launcher_pools))
+        self.assertEqual(launcher.id, list(launcher_pools)[0].id)
+
+        launcher.unregister()
+        launcher_pools = self.zk.getRegisteredPools()
+        self.assertEqual(0, len(launcher_pools))
 
 
 class TestZooKeeper(tests.DBTestCase):
@@ -28,43 +56,43 @@ class TestZooKeeper(tests.DBTestCase):
 
     def test_buildZooKeeperHosts_single(self):
         hosts = [
-            zk.ZooKeeperConnectionConfig('127.0.0.1', port=2181,
-                                         chroot='/test1')
+            ZooKeeperConnectionConfig('127.0.0.1', port=2181,
+                                      chroot='/test1')
         ]
         self.assertEqual('127.0.0.1:2181/test1',
-                         zk.buildZooKeeperHosts(hosts))
+                         buildZooKeeperHosts(hosts))
 
     def test_buildZooKeeperHosts_multiple(self):
         hosts = [
-            zk.ZooKeeperConnectionConfig('127.0.0.1', port=2181,
-                                         chroot='/test1'),
-            zk.ZooKeeperConnectionConfig('127.0.0.2', port=2182,
-                                         chroot='/test2')
+            ZooKeeperConnectionConfig('127.0.0.1', port=2181,
+                                      chroot='/test1'),
+            ZooKeeperConnectionConfig('127.0.0.2', port=2182,
+                                      chroot='/test2')
         ]
         self.assertEqual('127.0.0.1:2181/test1,127.0.0.2:2182/test2',
-                         zk.buildZooKeeperHosts(hosts))
+                         buildZooKeeperHosts(hosts))
 
     def test_buildZooKeeperHosts_ipv6(self):
         hosts = [
-            zk.ZooKeeperConnectionConfig(
+            ZooKeeperConnectionConfig(
                 '2001:4800:7817:103:be76:4eff:fe04:e359', port=2181,
                 chroot='/test1'),
-            zk.ZooKeeperConnectionConfig(
+            ZooKeeperConnectionConfig(
                 '[2002:4800:7817:103:be76:4eff:fe04:e359]', port=2181,
                 chroot='/test2'),
-            zk.ZooKeeperConnectionConfig('127.0.0.2', port=2182,
-                                         chroot='/test3')
+            ZooKeeperConnectionConfig('127.0.0.2', port=2182,
+                                      chroot='/test3')
         ]
         self.assertEqual((
             '[2001:4800:7817:103:be76:4eff:fe04:e359]:2181/test1,'
             '[2002:4800:7817:103:be76:4eff:fe04:e359]:2181/test2,'
             '127.0.0.2:2182/test3'
-        ), zk.buildZooKeeperHosts(hosts))
+        ), buildZooKeeperHosts(hosts))
 
     def test_imageBuildLock(self):
         path = self.zk._imageBuildLockPath("ubuntu-trusty")
         with self.zk.imageBuildLock("ubuntu-trusty", blocking=False):
-            self.assertIsNotNone(self.zk.client.exists(path))
+            self.assertIsNotNone(self.zk.kazoo_client.exists(path))
 
     def test_imageBuildLock_exception_nonblocking(self):
         image = "ubuntu-trusty"
@@ -87,7 +115,7 @@ class TestZooKeeper(tests.DBTestCase):
         with self.zk.imageBuildNumberLock(
             "ubuntu-trusty", "0000", blocking=False
         ):
-            self.assertIsNotNone(self.zk.client.exists(path))
+            self.assertIsNotNone(self.zk.kazoo_client.exists(path))
 
     def test_imageBuildNumberLock_exception_nonblocking(self):
         image = "ubuntu-trusty"
@@ -124,12 +152,12 @@ class TestZooKeeper(tests.DBTestCase):
                                                   upload.provider_name,
                                                   upload.id)
         with self.zk.imageUploadNumberLock(upload, blocking=False):
-            self.assertIsNotNone(self.zk.client.exists(path))
+            self.assertIsNotNone(self.zk.kazoo_client.exists(path))
             self.zk.deleteUpload(upload.image_name,
                                  upload.build_id,
                                  upload.provider_name,
                                  upload.id)
-        self.assertIsNone(self.zk.client.exists(path))
+        self.assertIsNone(self.zk.kazoo_client.exists(path))
 
     def test_imageUploadNumberLock_orphan(self):
         upload = zk.ImageUpload()
@@ -143,7 +171,7 @@ class TestZooKeeper(tests.DBTestCase):
                                                   upload.id)
 
         with self.zk.imageUploadNumberLock(upload, blocking=False):
-            self.assertIsNotNone(self.zk.client.exists(path))
+            self.assertIsNotNone(self.zk.kazoo_client.exists(path))
             self.zk.deleteUpload(upload.image_name,
                                  upload.build_id,
                                  upload.provider_name,
@@ -155,7 +183,7 @@ class TestZooKeeper(tests.DBTestCase):
             # We now recreated an empty image upload number node
             pass
 
-        self.assertIsNotNone(self.zk.client.exists(path))
+        self.assertIsNotNone(self.zk.kazoo_client.exists(path))
 
         # Should not throw an exception because of the empty upload
         self.zk.getImageUpload(upload.image_name, upload.build_id,
@@ -172,7 +200,7 @@ class TestZooKeeper(tests.DBTestCase):
                                                   upload.provider_name,
                                                   upload.id)
         with self.zk.imageUploadNumberLock(upload, blocking=False):
-            self.assertIsNotNone(self.zk.client.exists(path))
+            self.assertIsNotNone(self.zk.kazoo_client.exists(path))
 
     def test_imageUploadNumberLock_exception_nonblocking(self):
         upload = zk.ImageUpload()
@@ -204,7 +232,7 @@ class TestZooKeeper(tests.DBTestCase):
         path = self.zk._imageUploadLockPath("ubuntu-trusty", "0000", "prov1")
         with self.zk.imageUploadLock("ubuntu-trusty", "0000", "prov1",
                                      blocking=False):
-            self.assertIsNotNone(self.zk.client.exists(path))
+            self.assertIsNotNone(self.zk.kazoo_client.exists(path))
 
     def test_imageUploadLock_exception_nonblocking(self):
         image = "ubuntu-trusty"
@@ -320,12 +348,17 @@ class TestZooKeeper(tests.DBTestCase):
                          [upload_id])
 
     def test_build_request(self):
-        '''Test the build request API methods (has/submit/remove)'''
+        '''Test the build request API methods (has/get/submit/remove)'''
         image = "ubuntu-trusty"
         self.zk.submitBuildRequest(image)
         self.assertTrue(self.zk.hasBuildRequest(image))
+        build_request = self.zk.getBuildRequest(image)
+        self.assertEqual(build_request.image_name, image)
+        self.assertTrue(build_request.pending)
         self.zk.removeBuildRequest(image)
         self.assertFalse(self.zk.hasBuildRequest(image))
+        build_request = self.zk.getBuildRequest(image)
+        self.assertIsNone(build_request)
 
     def test_buildLock_orphan(self):
         image = "ubuntu-trusty"
@@ -339,7 +372,7 @@ class TestZooKeeper(tests.DBTestCase):
             # We now created an empty build number node
             pass
 
-        self.assertIsNotNone(self.zk.client.exists(path))
+        self.assertIsNotNone(self.zk.kazoo_client.exists(path))
 
         # Should not throw an exception because of the empty upload
         self.assertIsNone(self.zk.getBuild(image, build_number))
@@ -468,11 +501,15 @@ class TestZooKeeper(tests.DBTestCase):
         v3.state = zk.FAILED
         v4 = zk.ImageBuild()
         v4.state = zk.DELETING
-        self.zk.client.create(path + "/1", value=v1.serialize(), makepath=True)
-        self.zk.client.create(path + "/2", value=v2.serialize(), makepath=True)
-        self.zk.client.create(path + "/3", value=v3.serialize(), makepath=True)
-        self.zk.client.create(path + "/4", value=v4.serialize(), makepath=True)
-        self.zk.client.create(path + "/lock", makepath=True)
+        self.zk.kazoo_client.create(path + "/1", value=v1.serialize(),
+                                    makepath=True)
+        self.zk.kazoo_client.create(path + "/2", value=v2.serialize(),
+                                    makepath=True)
+        self.zk.kazoo_client.create(path + "/3", value=v3.serialize(),
+                                    makepath=True)
+        self.zk.kazoo_client.create(path + "/4", value=v4.serialize(),
+                                    makepath=True)
+        self.zk.kazoo_client.create(path + "/lock", makepath=True)
 
         matches = self.zk.getBuilds(image, None)
         self.assertEqual(4, len(matches))
@@ -488,11 +525,15 @@ class TestZooKeeper(tests.DBTestCase):
         v3.state = zk.FAILED
         v4 = zk.ImageBuild()
         v4.state = zk.DELETING
-        self.zk.client.create(path + "/1", value=v1.serialize(), makepath=True)
-        self.zk.client.create(path + "/2", value=v2.serialize(), makepath=True)
-        self.zk.client.create(path + "/3", value=v3.serialize(), makepath=True)
-        self.zk.client.create(path + "/4", value=v4.serialize(), makepath=True)
-        self.zk.client.create(path + "/lock", makepath=True)
+        self.zk.kazoo_client.create(path + "/1", value=v1.serialize(),
+                                    makepath=True)
+        self.zk.kazoo_client.create(path + "/2", value=v2.serialize(),
+                                    makepath=True)
+        self.zk.kazoo_client.create(path + "/3", value=v3.serialize(),
+                                    makepath=True)
+        self.zk.kazoo_client.create(path + "/4", value=v4.serialize(),
+                                    makepath=True)
+        self.zk.kazoo_client.create(path + "/lock", makepath=True)
 
         matches = self.zk.getBuilds(image, [zk.DELETING, zk.FAILED])
         self.assertEqual(2, len(matches))
@@ -507,11 +548,15 @@ class TestZooKeeper(tests.DBTestCase):
         v3.state = zk.FAILED
         v4 = zk.ImageUpload()
         v4.state = zk.DELETING
-        self.zk.client.create(path + "/1", value=v1.serialize(), makepath=True)
-        self.zk.client.create(path + "/2", value=v2.serialize(), makepath=True)
-        self.zk.client.create(path + "/3", value=v3.serialize(), makepath=True)
-        self.zk.client.create(path + "/4", value=v4.serialize(), makepath=True)
-        self.zk.client.create(path + "/lock", makepath=True)
+        self.zk.kazoo_client.create(path + "/1", value=v1.serialize(),
+                                    makepath=True)
+        self.zk.kazoo_client.create(path + "/2", value=v2.serialize(),
+                                    makepath=True)
+        self.zk.kazoo_client.create(path + "/3", value=v3.serialize(),
+                                    makepath=True)
+        self.zk.kazoo_client.create(path + "/4", value=v4.serialize(),
+                                    makepath=True)
+        self.zk.kazoo_client.create(path + "/lock", makepath=True)
 
         matches = self.zk.getUploads("trusty", "000", "rax",
                                      [zk.DELETING, zk.FAILED])
@@ -527,11 +572,15 @@ class TestZooKeeper(tests.DBTestCase):
         v3.state = zk.FAILED
         v4 = zk.ImageUpload()
         v4.state = zk.DELETING
-        self.zk.client.create(path + "/1", value=v1.serialize(), makepath=True)
-        self.zk.client.create(path + "/2", value=v2.serialize(), makepath=True)
-        self.zk.client.create(path + "/3", value=v3.serialize(), makepath=True)
-        self.zk.client.create(path + "/4", value=v4.serialize(), makepath=True)
-        self.zk.client.create(path + "/lock", makepath=True)
+        self.zk.kazoo_client.create(path + "/1", value=v1.serialize(),
+                                    makepath=True)
+        self.zk.kazoo_client.create(path + "/2", value=v2.serialize(),
+                                    makepath=True)
+        self.zk.kazoo_client.create(path + "/3", value=v3.serialize(),
+                                    makepath=True)
+        self.zk.kazoo_client.create(path + "/4", value=v4.serialize(),
+                                    makepath=True)
+        self.zk.kazoo_client.create(path + "/lock", makepath=True)
 
         matches = self.zk.getUploads("trusty", "000", "rax", None)
         self.assertEqual(4, len(matches))
@@ -559,26 +608,9 @@ class TestZooKeeper(tests.DBTestCase):
 
     def test_deleteUpload(self):
         path = self.zk._imageUploadPath("trusty", "000", "rax") + "/000001"
-        self.zk.client.create(path, makepath=True)
+        self.zk.kazoo_client.create(path, makepath=True)
         self.zk.deleteUpload("trusty", "000", "rax", "000001")
-        self.assertIsNone(self.zk.client.exists(path))
-
-    def test_registerLauncher(self):
-        launcher = zk.Launcher()
-        launcher.id = "launcher-Poolworker.provider-main-" + uuid.uuid4().hex
-        self.zk.registerLauncher(launcher)
-        launchers = self.zk.getRegisteredLaunchers()
-        self.assertEqual(1, len(launchers))
-        self.assertEqual(launcher.id, launchers[0].id)
-
-    def test_registerLauncher_safe_repeat(self):
-        launcher = zk.Launcher()
-        launcher.id = "launcher-Poolworker.provider-main-" + uuid.uuid4().hex
-        self.zk.registerLauncher(launcher)
-        self.zk.registerLauncher(launcher)
-        launchers = self.zk.getRegisteredLaunchers()
-        self.assertEqual(1, len(launchers))
-        self.assertEqual(launcher.id, launchers[0].id)
+        self.assertIsNone(self.zk.kazoo_client.exists(path))
 
     def test_getNodeRequests_empty(self):
         self.assertEqual([], self.zk.getNodeRequests())
@@ -588,10 +620,10 @@ class TestZooKeeper(tests.DBTestCase):
         r2 = self.zk._requestPath("100-456")
         r3 = self.zk._requestPath("100-123")
         r4 = self.zk._requestPath("400-123")
-        self.zk.client.create(r1, makepath=True, ephemeral=True)
-        self.zk.client.create(r2, makepath=True, ephemeral=True)
-        self.zk.client.create(r3, makepath=True, ephemeral=True)
-        self.zk.client.create(r4, makepath=True, ephemeral=True)
+        self.zk.kazoo_client.create(r1, makepath=True, ephemeral=True)
+        self.zk.kazoo_client.create(r2, makepath=True, ephemeral=True)
+        self.zk.kazoo_client.create(r3, makepath=True, ephemeral=True)
+        self.zk.kazoo_client.create(r4, makepath=True, ephemeral=True)
 
         self.assertEqual(
             ["100-123", "100-456", "400-123", "500-123"],
@@ -602,8 +634,8 @@ class TestZooKeeper(tests.DBTestCase):
         r = zk.NodeRequest("500-123")
         r.state = zk.REQUESTED
         path = self.zk._requestPath(r.id)
-        self.zk.client.create(path, value=r.serialize(),
-                              makepath=True, ephemeral=True)
+        self.zk.kazoo_client.create(path, value=r.serialize(),
+                                    makepath=True, ephemeral=True)
         o = self.zk.getNodeRequest(r.id)
         self.assertIsInstance(o, zk.NodeRequest)
         self.assertEqual(r.id, o.id)
@@ -612,8 +644,8 @@ class TestZooKeeper(tests.DBTestCase):
         self.assertIsNone(self.zk.getNodeRequest("invalid"))
 
     def test_getNodes(self):
-        self.zk.client.create(self.zk._nodePath('100'), makepath=True)
-        self.zk.client.create(self.zk._nodePath('200'), makepath=True)
+        self.zk.kazoo_client.create(self.zk._nodePath('100'), makepath=True)
+        self.zk.kazoo_client.create(self.zk._nodePath('200'), makepath=True)
         nodes = self.zk.getNodes()
         self.assertIn('100', nodes)
         self.assertIn('200', nodes)
@@ -622,7 +654,7 @@ class TestZooKeeper(tests.DBTestCase):
         n = zk.Node('100')
         n.state = zk.BUILDING
         path = self.zk._nodePath(n.id)
-        self.zk.client.create(path, value=n.serialize(), makepath=True)
+        self.zk.kazoo_client.create(path, value=n.serialize(), makepath=True)
         o = self.zk.getNode(n.id)
         self.assertIsInstance(o, zk.Node)
         self.assertEqual(n.id, o.id)
@@ -643,7 +675,7 @@ class TestZooKeeper(tests.DBTestCase):
         self.zk.lockNode(node)
         self.assertIsNotNone(node.lock)
         self.assertIsNotNone(
-            self.zk.client.exists(self.zk._nodeLockPath(node.id))
+            self.zk.kazoo_client.exists(self.zk._nodeLockPath(node.id))
         )
         self.zk.unlockNode(node)
         self.assertIsNone(node.lock)
@@ -662,7 +694,7 @@ class TestZooKeeper(tests.DBTestCase):
         self.zk.storeNode(node)
         self.assertIsNotNone(node.id)
         self.assertIsNotNone(
-            self.zk.client.exists(self.zk._nodePath(node.id))
+            self.zk.kazoo_client.exists(self.zk._nodePath(node.id))
         )
         return node
 
@@ -684,7 +716,7 @@ class TestZooKeeper(tests.DBTestCase):
         req.node_types.append('label1')
         self.zk.storeNodeRequest(req)
         self.assertIsNotNone(
-            self.zk.client.exists(self.zk._requestPath(req.id))
+            self.zk.kazoo_client.exists(self.zk._requestPath(req.id))
         )
         return req
 
@@ -706,14 +738,14 @@ class TestZooKeeper(tests.DBTestCase):
         req = self._create_node_request()
         self.zk.deleteNodeRequest(req)
         self.assertIsNone(
-            self.zk.client.exists(self.zk._requestPath(req.id))
+            self.zk.kazoo_client.exists(self.zk._requestPath(req.id))
         )
 
     def test_deleteNode(self):
         n1 = self._create_node()
         self.zk.deleteNode(n1)
         self.assertIsNone(
-            self.zk.client.exists(self.zk._nodePath(n1.id))
+            self.zk.kazoo_client.exists(self.zk._nodePath(n1.id))
         )
 
     def test_getReadyNodesOfTypes(self):
